/*
 * @Author: your name
 * @Date: 2022-03-19 22:10:39
 * @LastEditTime: 2022-03-21 19:33:21
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath: /roadmodel/mpc_drv_ctap_rm/core/road/delay.h
 */
#pragma once

#include "global_parameters.h"
#include <algorithm>
#include <cmath>
#include <iostream>

// the rising edge is delayed by risingtime, and the falling edge is delayed by
// fallingtime
namespace rm {
template <typename Type> class Delay {
public:
  Delay(){};
  Delay(float risingtime, float fallingtime, Type default_state)
      : rising_cycle_(std::max(static_cast<uint32_t>(std::floor(
                                   risingtime / rm::ROADMODELCYCLEINS)),
                               static_cast<uint32_t>(0))),
        falling_cycle_(std::max(static_cast<uint32_t>(std::floor(
                                    fallingtime / rm::ROADMODELCYCLEINS)),
                                static_cast<uint32_t>(0))),
        default_state_(default_state){};
  ~Delay(){};
  void SetRiseDelayTime(const float risingtime) {
    rising_cycle_ = std::max(
        static_cast<uint32_t>(std::floor(risingtime / rm::ROADMODELCYCLEINS)),
        static_cast<uint32_t>(0));
  };
  void SetFallDelayTime(const float fallingtime) {
    falling_cycle_ = std::max(
        static_cast<uint32_t>(std::floor(fallingtime / rm::ROADMODELCYCLEINS)),
        static_cast<uint32_t>(0));
  };
  Type RunDelay(const Type flag) {
    if (first_run_ == true) {
      first_run_ = false;
      if (rising_cycle_ == 0U) {
        state_prev_ = flag;
      } else {
        state_prev_ = default_state_;
      }
      return state_prev_;
    } else {
      return DelayCounter(flag);
    }
  };
  void ResetDelay() {
    first_run_ = true;
    cycle_ = 0;
    state_prev_ = default_state_;
  }

private:
  Type DelayCounter(const Type flag) {
    if (flag != state_prev_) {
      int cycle_threshold =
          (flag < state_prev_) ? falling_cycle_ : rising_cycle_;
      cycle_++;
      if (cycle_ >= cycle_threshold) {
        state_prev_ = flag;
        cycle_ = 0;
        return state_prev_;
      } else {
        return state_prev_;
      }
    } else {
      cycle_ = 0;
      return state_prev_;
    }
  };
  bool first_run_ = true;
  uint32_t cycle_ = 0;
  uint32_t rising_cycle_ = 0;
  uint32_t falling_cycle_ = 0;
  Type state_prev_;
  Type default_state_;
}; // namespace rm

} // namespace rm